package upgrade.test.presentation;

import com.vaadin.Application;
import com.vaadin.terminal.Sizeable;
import com.vaadin.terminal.Terminal;
import com.vaadin.terminal.gwt.server.HttpServletRequestListener;
import com.vaadin.ui.*;
import com.vaadin.ui.Window.Notification;
import java.io.PrintWriter;
import java.io.Serializable;
import java.io.StringWriter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import upgrade.test.ejb.HelloBeanLocal;
import upgrade.test.ejb.HelloBeanRemote;
import upgrade.test.ejb.JPABeanLocal;
import upgrade.test.tests.*;

/**
 * Vaadin application object that represents the web app. For more info
 * see http://vaadin.com
 *
 * JavaDocs: http://vaadin.com/api/
 *
 * @author Bobby Bissett
 */
public class WebApp extends Application
    implements Serializable, HttpServletRequestListener {

    private static final boolean addNegTest = false;

    // passed in from Servlet
    private HelloBeanLocal helloBean;
    private HelloBeanRemote helloBeanRemote;
    private JPABeanLocal jpaBean;

    private TextField outputField;

    // set at the beginning of each request through HttpServletRequestListener
    private HttpServletRequest request;

    // constructor called by servler to pass resources
    public WebApp(HelloBeanLocal helloBean,
        HelloBeanRemote helloBeanRemote, JPABeanLocal jpaBean) {
        this.helloBean = helloBean;
        this.helloBeanRemote = helloBeanRemote;
        this.jpaBean= jpaBean;
    }

    @Override
    public void init() {
//        setTheme("runo");

        Window window = new Window("Upgrade Test Application");
        setMainWindow(window);

        // divide page into left/right and center in browser
        HorizontalLayout mainPage = new HorizontalLayout();
        VerticalLayout root = new VerticalLayout();
        window.setContent(root);
        root.addComponent(mainPage);
        root.setComponentAlignment(mainPage, Alignment.TOP_CENTER);

        // add panels
        Panel left = new Panel("Tests");
        left.setWidth(50, Sizeable.UNITS_EM);
        mainPage.addComponent(left);
        Panel right = new Panel("Test Output");
        right.setWidth(40, Sizeable.UNITS_EM);
        mainPage.addComponent(right);

        // fill in left side
        left.addComponent(new LoginTest(this));
        left.addComponent(new HelloEjbTest(this));
        left.addComponent(new JPATest(this));
        if (addNegTest) {
            left.addComponent(new NegativeTest(this));
        }

        // fill in right side
        outputField = new TextField();
        outputField.setWidth("100%");
        outputField.setRows(25);
        right.addComponent(outputField);
    }

    /*
     * We will override this method to put error information in a useful
     * place in the web page. By calling super.terminalError(ee) as well,
     * the Vaadin runtime will mark the button that was clicked to cause
     * the error along with a rollover message. We won't need that since
     * we're explicitly showing the stack trace in the text field.
     */
    @SuppressWarnings({"ThrowableResultOfMethodCallIgnored"})
    @Override
    public void terminalError(Terminal.ErrorEvent ee) {
        Throwable cause = ee.getThrowable().getCause();
        if (cause == null) {
            cause = ee.getThrowable();
        }

        StringWriter sWriter = new StringWriter();
        cause.printStackTrace(new PrintWriter(sWriter));
        outputField.setValue(sWriter.toString());
        getMainWindow().showNotification(cause.getMessage(),
            "See output panel or server log for more information",
            Notification.TYPE_ERROR_MESSAGE);
    }

    @Override
    public void onRequestStart(HttpServletRequest request,
        HttpServletResponse response) {
        this.request = request;
    }

    @Override
    public void onRequestEnd(HttpServletRequest request,
        HttpServletResponse response) {
        this.request = null;
    }

    public HttpServletRequest getRequest() {
        return request;
    }

    public HelloBeanLocal getHelloBean() {
        return helloBean;
    }

    public HelloBeanRemote getHelloBeanRemote() {
        return helloBeanRemote;
    }

    public JPABeanLocal getJpaBean() {
        return jpaBean;
    }

    // helper method for a warning notification
    public void warn(String msg) {
        doNotify(msg, Notification.TYPE_WARNING_MESSAGE);
    }

    // helper method for info notification
    public void info(String msg) {
        doNotify(msg, Notification.TYPE_HUMANIZED_MESSAGE);
    }

    private void doNotify(String msg, int type) {
        outputField.setValue(String.format("[%s]", msg));
        getMainWindow().showNotification(msg, type);
    }
}
