package upgrade.test.tests;

import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import upgrade.test.presentation.WebApp;

/*
 * Simple base class for tests.
 */
public abstract class AbstractTestPanel extends Panel {

    WebApp app;
    Label title;

    /*
     * Subclasses should have a simple constructor that just
     * sets the WebApp class. All other initialization code
     * should be in the attach() method.
     */
    public AbstractTestPanel(WebApp app) {
        super(); // may want to pass title here
        this.app = app;
        title = new Label(getTitle());
        addComponent(title);
        addComponent(new Label(getTestDescription()));
    }

    void pass(String msg) {
        app.info(msg);
        swapTitles("#00ff00");
    }

    void fail(String msg) {
        System.err.println(String.format(
            "%s: %s", getTitle(), msg));
        app.warn(msg);
        swapTitles("#ff0000");
    }

    // creates a bold title of the given color and replaces the old one
    private void swapTitles(String color) {
        String value = String.format("<font color=\"%s\"><b>%s</b></font>",
            color, getTitle());
        Label newTitle = new Label(value, Label.CONTENT_XHTML);
        replaceComponent(title, newTitle);
        title = newTitle;
    }

    /*
     * Am forcing subclasses to implement this method to
     * do things like add buttons and then selves as listeners.
     * This way we avoid partially constructed panels escaping
     * into the wild.
     */
    @Override
    abstract public void attach();

    abstract String getTitle();
    abstract String getTestDescription();
}
