package upgrade.test.tests;

import com.vaadin.ui.*;
import upgrade.test.ejb.HelloBeanLocal;
import upgrade.test.ejb.HelloBeanRemote;
import upgrade.test.presentation.WebApp;

public class HelloEjbTest extends AbstractTestPanel
    implements Button.ClickListener {

    private boolean localPass = false;
    private Button localButton = new Button("Local");
    private Button remoteButton = new Button("Remote");

    public HelloEjbTest(WebApp app) {
        super(app);
    }

    @Override
    String getTitle() {
        return "Hello Ejb Test";
    }

    @Override
    String getTestDescription() {
        return "Test of stateless EJB local and remote invocations. " +
            "Click both buttons to test.";
    }

    @Override
    public void attach() {
        HorizontalLayout hl = new HorizontalLayout();
        hl.setSpacing(true);
        localButton.addListener(this);
        remoteButton.addListener(this);
        remoteButton.setEnabled(false);
        hl.addComponent(localButton);
        hl.addComponent(remoteButton);
        addComponent(hl);
    }

    @Override
    public void buttonClick(Button.ClickEvent clickEvent) {
        if (clickEvent.getButton() == localButton) {
            HelloBeanLocal bean = app.getHelloBean();
            final String in = "foo";
            final String out = "Hello foo";
            String result = bean.hello(in);
            if (out.equals(result)) {
                localPass = true;
                app.info("Local invocation passed");
            } else {
                String msg = String.format(
                    "Local invocation failed. Expected '%s' and received '%s'",
                    out, result);
                fail(msg);
            }
            localButton.setEnabled(false);
            remoteButton.setEnabled(true);
        } else {
            HelloBeanRemote bean = app.getHelloBeanRemote();
            final String in = "bar";
            final String out = "Remote hello bar";
            String result = bean.helloRemote(in);
            if (out.equals(result)) {
                if (localPass) {
                    pass("Both tests passed");
                }
            } else {
                String msg = String.format(
                    "Remote invocation failed. Expected '%s' and received '%s'",
                    out, result);
                fail(msg);
            }
            localButton.setEnabled(true);
            remoteButton.setEnabled(false);
        }
    }
}
