package upgrade.test.tests;

import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import upgrade.test.ejb.JPABeanLocal;
import upgrade.test.pojo.MyPojo;
import upgrade.test.presentation.WebApp;

public class JPATest extends AbstractTestPanel implements Button.ClickListener {

    private Button button1 = new Button("Test Create/Retrieve");
    private Button button2 = new Button("Test Expected Exception");
    private MyPojo myPojo;

    public JPATest(WebApp app) {
        super(app);
        myPojo = new MyPojo();
        myPojo.setName("Homer");
        myPojo.setNumber(123);
    }

    @Override
    public void attach() {
        HorizontalLayout hl = new HorizontalLayout();
        hl.setSpacing(true);
        button1.addListener(this);
        button2.addListener(this);
        button2.setEnabled(false);
        hl.addComponent(button1);
        hl.addComponent(button2);
        addComponent(hl);
    }

    @Override
    String getTitle() {
        return "JPA Test";
    }

    @Override
    String getTestDescription() {
        return "This is a test that uses JPA in an entity bean to create " +
            "and retrieve a simple pojo. The persistence.xml file instructs " +
            "the provide to drop/recreate tables upon deployment, so the " +
            "first button should create/retrieve/match the pojo and the  " +
            "test should pass. The second button should cause a unique " +
            "constraint violation to be thrown, or else the test will fail. " +
            "These tests can only be run once after deployment.";
    }

    @Override
    public void buttonClick(Button.ClickEvent clickEvent) {
        if (clickEvent.getButton() == button1) {
            JPABeanLocal bean = app.getJpaBean();
            bean.createMP(myPojo);
            final long id = myPojo.getId();

            MyPojo newMP = bean.getMPByID(id);
            if (myPojo.equals(newMP)) {
                pass("Create, retrieve, and match passed.");
                button1.setEnabled(false);
                button2.setEnabled(true);
            } else {
                String msg = String.format(
                    "%s failed. Expected '%s' and received '%s'",
                    getTitle(), myPojo, newMP);
                fail(msg);
            }
        } else {
            button2.setEnabled(false);
            MyPojo badPojo = new MyPojo();
            badPojo.setName(myPojo.getName());
            badPojo.setNumber(myPojo.getNumber());

            JPABeanLocal bean = app.getJpaBean();
            
            bean.createMP(badPojo);
            fail("Did not receive expected exception saving pojo with " +
                "same name as existing pojo.");
        }
    }
}
